class User < ActiveRecord::Base
  attr_accessor :password
  before_save :encrypt_password
  has_many :posts
  has_many :comments, class_name: "Post", foreign_key: :author_id
  has_and_belongs_to_many :friends,
    class_name: 'User',
    foreign_key: 'friend_id',
    associated_foreign_key: 'user_id',
    join_table: 'friends_users'
  
  validates :email,
    {
      presence: true,
      uniqueness: true
    }
  validates :name, presence: true
  validates :password, confirmation: true, :if => :password_required?
  
  def self.encrypt(pass, salt)
    Digest::SHA2.hexdigest(salt+pass)
  end

  def encrypt_password
    return if password.blank?
    if new_record?
        self.salt = SecureRandom.base64(8)
    end
    self.encrypted_password = User.encrypt(password, salt)
  end

  def self.authenticate(email, pass)
    user = User.where(email: email).take
    # user = User.find_by username: user
    user && user.authenticated?(pass) ? user : nil
  end

  def authenticated?(pass)
    encrypted_password == User.encrypt(pass, salt)
  end

  def password_required?
    !password.blank? || encrypted_password.blank? # new_record?
  end

  def get_posts_page(page)
      self.posts.order('updated_at desc').paginate(page: page, per_page: 5)
  end
end
